(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.Designable = global.Designable || {}, global.Designable.Vue = {})));
})(this, (function (exports) { 'use strict';

    /******************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise, SuppressedError, Symbol */


    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
        var e = new Error(message);
        return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
    };

    var DesignerEngineContext = Symbol('DesignerEngineContext');
    var DesignerComponentsContext = Symbol('DesignerComponentsContext');
    var TreeNodeContext = Symbol('TreeNodeContext');

    function useDesigner() {
        var designer = Vue.inject(DesignerEngineContext);
        return designer;
    }

    function useComponents() {
        var components = Vue.inject(DesignerComponentsContext);
        return components;
    }

    function usePrefix(after) {
        if (after === void 0) { after = ''; }
        return "dn-".concat(after);
    }

    function useTreeNode() {
        var designer = Vue.inject(TreeNodeContext);
        return designer;
    }

    var TreeNodeWidget = Formily.ReactiveVue.observer(Vue.defineComponent({
        name: 'TreeNodeWidget',
        props: {
            node: {
                type: Object,
                required: true,
            },
        },
        setup: function (props) {
            var designer = useDesigner();
            var components = useComponents();
            var nodeRef = Vue.computed(function () { return props.node; });
            Vue.provide(TreeNodeContext, nodeRef);
            return function () {
                var node = props.node;
                var renderChildren = function () {
                    var _a, _b;
                    if ((_a = node === null || node === void 0 ? void 0 : node.designerProps) === null || _a === void 0 ? void 0 : _a.selfRenderChildren)
                        return [];
                    return (_b = node === null || node === void 0 ? void 0 : node.children) === null || _b === void 0 ? void 0 : _b.map(function (child) {
                        return Vue.h(TreeNodeWidget, { node: child, key: child.id });
                    });
                };
                var renderProps = function (extendsProps) {
                    var _a, _b, _c;
                    if (extendsProps === void 0) { extendsProps = {}; }
                    var props = __assign(__assign(__assign(__assign({}, (_a = node.designerProps) === null || _a === void 0 ? void 0 : _a.defaultProps), extendsProps), node.props), (_c = (_b = node.designerProps) === null || _b === void 0 ? void 0 : _b.getComponentProps) === null || _c === void 0 ? void 0 : _c.call(_b, node));
                    if (node.depth === 0) {
                        delete props.style;
                    }
                    return props;
                };
                var renderComponent = function () {
                    var _a, _b, _c;
                    var componentName = node.componentName;
                    var Component = (_a = components.value) === null || _a === void 0 ? void 0 : _a[componentName];
                    var dataId = {};
                    if (Component) {
                        if (designer.value) {
                            dataId[(_b = designer.value.props) === null || _b === void 0 ? void 0 : _b.nodeIdAttrName] = node.id;
                        }
                        return Vue.h(Component, renderProps(dataId), {
                            default: renderChildren,
                        });
                    }
                    else {
                        if ((_c = node === null || node === void 0 ? void 0 : node.children) === null || _c === void 0 ? void 0 : _c.length) {
                            return Vue.h(Vue.Fragment, renderChildren());
                        }
                    }
                };
                if (!node)
                    return null;
                if (node.hidden)
                    return null;
                return Vue.h(Vue.Fragment, [renderComponent()]);
            };
        },
    }));
    var ComponentTreeWidget = Formily.ReactiveVue.observer(Vue.defineComponent({
        name: 'ComponentTreeWidget',
        props: {
            components: {
                type: Object,
                required: true,
            },
            tree: {
                type: Object,
                required: true,
            },
            designer: {
                type: Object,
                required: true,
            },
        },
        setup: function (props) {
            var treeRef = Vue.computed(function () { return props.tree; });
            Vue.provide(TreeNodeContext, treeRef);
            var componentsRef = Vue.computed(function () { return props.components; });
            Vue.provide(DesignerComponentsContext, componentsRef);
            var designerRef = Vue.computed(function () { return props.designer; });
            Vue.provide(DesignerEngineContext, designerRef);
            Vue.watch(function () { return props.components; }, function () {
                Designable.Core.GlobalRegistry.registerDesignerBehaviors(props.components);
            }, { immediate: true });
            return function () {
                var tree = props.tree;
                return Vue.h(Vue.Fragment, null, [Vue.h(TreeNodeWidget, { node: tree })]);
            };
        },
    }));

    exports.ComponentTreeWidget = ComponentTreeWidget;
    exports.DesignerComponentsContext = DesignerComponentsContext;
    exports.DesignerEngineContext = DesignerEngineContext;
    exports.TreeNodeContext = TreeNodeContext;
    exports.TreeNodeWidget = TreeNodeWidget;
    exports.useComponents = useComponents;
    exports.useDesigner = useDesigner;
    exports.usePrefix = usePrefix;
    exports.useTreeNode = useTreeNode;

}));
//# sourceMappingURL=designable.vue.umd.production.js.map
