var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { GlobalRegistry } from '@antdev/designable-core';
import { observer } from '@formily/reactive-vue';
import { computed, defineComponent, Fragment, h, provide, watch, } from 'vue';
import { DesignerComponentsContext, DesignerEngineContext, TreeNodeContext, } from '../../context';
import { useComponents, useDesigner } from '../../hooks';
export var TreeNodeWidget = observer(defineComponent({
    name: 'TreeNodeWidget',
    props: {
        node: {
            type: Object,
            required: true,
        },
    },
    setup: function (props) {
        var designer = useDesigner();
        var components = useComponents();
        var nodeRef = computed(function () { return props.node; });
        provide(TreeNodeContext, nodeRef);
        return function () {
            var node = props.node;
            var renderChildren = function () {
                var _a, _b;
                if ((_a = node === null || node === void 0 ? void 0 : node.designerProps) === null || _a === void 0 ? void 0 : _a.selfRenderChildren)
                    return [];
                return (_b = node === null || node === void 0 ? void 0 : node.children) === null || _b === void 0 ? void 0 : _b.map(function (child) {
                    return h(TreeNodeWidget, { node: child, key: child.id });
                });
            };
            var renderProps = function (extendsProps) {
                var _a, _b, _c;
                if (extendsProps === void 0) { extendsProps = {}; }
                var props = __assign(__assign(__assign(__assign({}, (_a = node.designerProps) === null || _a === void 0 ? void 0 : _a.defaultProps), extendsProps), node.props), (_c = (_b = node.designerProps) === null || _b === void 0 ? void 0 : _b.getComponentProps) === null || _c === void 0 ? void 0 : _c.call(_b, node));
                if (node.depth === 0) {
                    delete props.style;
                }
                return props;
            };
            var renderComponent = function () {
                var _a, _b, _c;
                var componentName = node.componentName;
                var Component = (_a = components.value) === null || _a === void 0 ? void 0 : _a[componentName];
                var dataId = {};
                if (Component) {
                    if (designer.value) {
                        dataId[(_b = designer.value.props) === null || _b === void 0 ? void 0 : _b.nodeIdAttrName] = node.id;
                    }
                    return h(Component, renderProps(dataId), {
                        default: renderChildren,
                    });
                }
                else {
                    if ((_c = node === null || node === void 0 ? void 0 : node.children) === null || _c === void 0 ? void 0 : _c.length) {
                        return h(Fragment, renderChildren());
                    }
                }
            };
            if (!node)
                return null;
            if (node.hidden)
                return null;
            return h(Fragment, [renderComponent()]);
        };
    },
}));
export var ComponentTreeWidget = observer(defineComponent({
    name: 'ComponentTreeWidget',
    props: {
        components: {
            type: Object,
            required: true,
        },
        tree: {
            type: Object,
            required: true,
        },
        designer: {
            type: Object,
            required: true,
        },
    },
    setup: function (props) {
        var treeRef = computed(function () { return props.tree; });
        provide(TreeNodeContext, treeRef);
        var componentsRef = computed(function () { return props.components; });
        provide(DesignerComponentsContext, componentsRef);
        var designerRef = computed(function () { return props.designer; });
        provide(DesignerEngineContext, designerRef);
        watch(function () { return props.components; }, function () {
            GlobalRegistry.registerDesignerBehaviors(props.components);
        }, { immediate: true });
        return function () {
            var tree = props.tree;
            return h(Fragment, null, [h(TreeNodeWidget, { node: tree })]);
        };
    },
}));
