"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentTreeWidget = exports.TreeNodeWidget = void 0;
var designable_core_1 = require("@antdev/designable-core");
var reactive_vue_1 = require("@formily/reactive-vue");
var vue_1 = require("vue");
var context_1 = require("../../context");
var hooks_1 = require("../../hooks");
exports.TreeNodeWidget = (0, reactive_vue_1.observer)((0, vue_1.defineComponent)({
    name: 'TreeNodeWidget',
    props: {
        node: {
            type: Object,
            required: true,
        },
    },
    setup: function (props) {
        var designer = (0, hooks_1.useDesigner)();
        var components = (0, hooks_1.useComponents)();
        var nodeRef = (0, vue_1.computed)(function () { return props.node; });
        (0, vue_1.provide)(context_1.TreeNodeContext, nodeRef);
        return function () {
            var node = props.node;
            var renderChildren = function () {
                var _a, _b;
                if ((_a = node === null || node === void 0 ? void 0 : node.designerProps) === null || _a === void 0 ? void 0 : _a.selfRenderChildren)
                    return [];
                return (_b = node === null || node === void 0 ? void 0 : node.children) === null || _b === void 0 ? void 0 : _b.map(function (child) {
                    return (0, vue_1.h)(exports.TreeNodeWidget, { node: child, key: child.id });
                });
            };
            var renderProps = function (extendsProps) {
                var _a, _b, _c;
                if (extendsProps === void 0) { extendsProps = {}; }
                var props = __assign(__assign(__assign(__assign({}, (_a = node.designerProps) === null || _a === void 0 ? void 0 : _a.defaultProps), extendsProps), node.props), (_c = (_b = node.designerProps) === null || _b === void 0 ? void 0 : _b.getComponentProps) === null || _c === void 0 ? void 0 : _c.call(_b, node));
                if (node.depth === 0) {
                    delete props.style;
                }
                return props;
            };
            var renderComponent = function () {
                var _a, _b, _c;
                var componentName = node.componentName;
                var Component = (_a = components.value) === null || _a === void 0 ? void 0 : _a[componentName];
                var dataId = {};
                if (Component) {
                    if (designer.value) {
                        dataId[(_b = designer.value.props) === null || _b === void 0 ? void 0 : _b.nodeIdAttrName] = node.id;
                    }
                    return (0, vue_1.h)(Component, renderProps(dataId), {
                        default: renderChildren,
                    });
                }
                else {
                    if ((_c = node === null || node === void 0 ? void 0 : node.children) === null || _c === void 0 ? void 0 : _c.length) {
                        return (0, vue_1.h)(vue_1.Fragment, renderChildren());
                    }
                }
            };
            if (!node)
                return null;
            if (node.hidden)
                return null;
            return (0, vue_1.h)(vue_1.Fragment, [renderComponent()]);
        };
    },
}));
exports.ComponentTreeWidget = (0, reactive_vue_1.observer)((0, vue_1.defineComponent)({
    name: 'ComponentTreeWidget',
    props: {
        components: {
            type: Object,
            required: true,
        },
        tree: {
            type: Object,
            required: true,
        },
        designer: {
            type: Object,
            required: true,
        },
    },
    setup: function (props) {
        var treeRef = (0, vue_1.computed)(function () { return props.tree; });
        (0, vue_1.provide)(context_1.TreeNodeContext, treeRef);
        var componentsRef = (0, vue_1.computed)(function () { return props.components; });
        (0, vue_1.provide)(context_1.DesignerComponentsContext, componentsRef);
        var designerRef = (0, vue_1.computed)(function () { return props.designer; });
        (0, vue_1.provide)(context_1.DesignerEngineContext, designerRef);
        (0, vue_1.watch)(function () { return props.components; }, function () {
            designable_core_1.GlobalRegistry.registerDesignerBehaviors(props.components);
        }, { immediate: true });
        return function () {
            var tree = props.tree;
            return (0, vue_1.h)(vue_1.Fragment, null, [(0, vue_1.h)(exports.TreeNodeWidget, { node: tree })]);
        };
    },
}));
